/*

	FileName: 		CmnDlgs.cc
   Author:			Gary White
   Written for:	Visual dBASE 7.x / dBASE 2000

   Version History:
      07/12/1999	Original Version
      07/17/1999	Version 1.1
      					No modifications to this cc (except these notes)
                     Modified the CmdDlgs.dll NewDir form
                     	Properly specify the current directory
                        Removed minimize/maximize buttons
                        Constrained resizing
                     Modified the GetDirectory form
                     	When resized, the two panes now retain their
	                        proportional sizes.
                     Modified the OpenDialog and SaveDialog
                     	Restores the directory before exiting
      07/24/1999	Version 1.2
      					Added DateDialog class with corresponding addition
                     to the CmnDlgs.dll.

   Description: 	Provides a wrapper to allow access to some common dialogs
   				 	surfaced in the CmnDlgs.dll.

   Dependencies:	CmnDlgs.dll (included in the zip file this CC came in.
						This file (CmnDlgs.cc) must be included in your EXE and
                  the DLL (CmnDlgs.dll) must be deployed with your 
                  application.  It may be installed in the same directory
                  as your executable, or in the Windows System directory.

   Copyright:		This file is copyright 1999, Gary White, all rights reserved
   					You are free to use this in your dBASE application and to
                  distribute the included DLL (CmnDlgs.dll) with your dBASE
                  application.  You may not sell these files in any manner
                  whereby they are not included in an application.

                  This code is not warranteed in any way.  Your use of these
                  files releases Gary White of all liability.
*/

class OpenDialog of FileDialog
/*
	Wrapper for the GetOpenFileName common dialog
   See the base FileDialog class for more details on properties

   Implementation:
   					set procedure to CmnDlgs.cc additive
                  d = new OpenDialog()
                  d.fileName = 'c:\Some directory\Default.txt'
                  d.addFilter('Text files (*.txt)|*.txt')
                  d.addFilter('All files (*.*)|*.*')
                  if d.execute()
                  	cFileName = d.fileName
                     // do whatever
                  else
                  	// user canceled
                  endif
                  d.release()

*/

	this.titleText := 'Open'	// default title bar text

   function execute
   	// returns true if the user chose a valid file
      // or false if user canceled.  Filename chosen
      // is stored in the FileName property.
   	super::execute()
   	with ( this )
      	fileName := GetFileName(fileName, ;
         								filter, ;
                                 titleText, ;
                                 defaultExt, ;
                                 filterIndex )
      endwith
		return not empty(this.fileName)

endclass

class SaveAsDialog of FileDialog
/*
	Wrapper for the GetSaveFileName common dialog
   See the base FileDialog class for more details on properties

   Implementation:
   					set procedure to CmnDlgs.cc additive
                  d = new SaveAsDialog()
                  d.fileName = 'c:\Some directory\Default.txt'
                  d.addFilter('Text files (*.txt)|*.txt')
                  d.addFilter('All files (*.*)|*.*')
                  if d.execute()
                  	cFileName = d.fileName
                     // do whatever
                  else
                  	// user canceled
                  endif
                  d.release()

*/
	this.titleText := 'Save As'	// default title bar text

   function execute
   	// returns true if the user chose a valid file
      // or false if user canceled.  Filename chosen
      // is stored in the FileName property.
   	super::execute()
   	with ( this )
      	fileName := PutFileName(fileName, ;
         								filter, ;
                                 titleText, ;
                                 defaultExt, ;
                                 filterIndex )
      endwith
		return not empty(this.fileName)

endclass

class FileDialog
	// generic base class for the OpenDialog and SaveAsDialog classes
	protect Init

	this.fileName = ''	// default filename to appear in the entryfield
   							// if you include the path, that will be the 
                        // starting path for the dialog
   this.filter = ''		// filter string (see addFilter method)
   this.filterIndex = 0	// which filter will be selected to start
   this.titleText = ''	// title bar text
   this.defaultExt = ''	// default extension if the user doesn't specify one

   this.Init()				// prototype the extern calls

   function addFilter(cFilter)

   	// adds a filter to the filter property
      // filters take the format:
      //    string to display separated by a vertical bar from the filter
      //		the text left of the vertical bar is displayed in the combobox
      //		the text right of the vertical bar is the actual filter
      // example:
      //    'All files (*.*)|*.*'

   	if '|' $ cFilter		// a legal filter will have a vertical bar
         if len(this.filter)== 0		// starting from scratch
            this.filter := cFilter
         else								// add to existing filter
            this.filter += '|' + cFilter
         endif
      else
      	msgbox('Bad filter format','Error',16)
      endif
      return

   function execute
   	if empty(this.filter)		// if we don't have a filter
      	this.filter := 'All files (*.*)|*.*'	// default to all files
      endif
      if this.filterIndex == 0	// if the filterIndex isn't set
         this.filterIndex := 1	// default to the first one.
      endif
      return

   function Init
      if type('GetFileName') # 'FP'
         extern CSTRING GetFileName( CSTRING, CSTRING, CSTRING, CSTRING, CINT ) CmnDlgs
      endif
      if type('PutFileName') # 'FP'
         extern CSTRING PutFileName( CSTRING, CSTRING, CSTRING, CSTRING, CINT ) CmnDlgs
      endif
      return

   function release
   	release object this
      return

endclass

class DateDialog
/*
	Wrapper for GetDateValue function from CmnDlgs.dll
   Displays a calendar and allows the user to select a date
   Valid dates range from Jan 1, 1000 to Dec 31, 2500

   Implementation:
   	set procedure to CmnDlgs.cc additive
      d = new DateDialog()
      d.defaultDate := {01/01/2000}
      if d.execute()
      	dMyDate := d.dateValue
      else
      	// user canceled
      endif
      d.release()

*/
	protect Init

   this.dateValue = {}		// the value returned from the dll will be stored here
   this.defaultDate = {}	// date to start the calendar on
   this.Init()

   function Init
   	if type('GetDateValue') # 'FP'
      	extern CLOGICAL GetDateValue( CPTR CUSHORT,CPTR CUSHORT,CPTR CUSHORT ) CmnDlgs
      endif
      return

   function execute
   	local y, m, d
   	if empty(this.defaultDate)					// if we don't have a default date
      	this.defaultDate = new date()			// use the current system date
      elseif type('this.defaultDate')=='C'	// if we got a string
      	try											// try for a type conversion
      		this.defaultDate = new date(this.defaultDate)
			catch (exception e)
         	thid.defaultDate = new date()
         endtry
      endif
      // now parse out the year, month and day from the default date
      y= this.defaultDate.GetYear()		// returns 2 digit year for < 2000
      if y<1000
      	y += 1900
      endif
      if y > 2500								// calendar won't handle year > 2500
      	y := 2500
      endif
      m= this.defaultDate.GetMonth() + 1	// GetMonth() returns zero based month
      d= this.defaultDate.GetDate()
      this.dateValue = {}								// initialize the return value
      if GetDateValue(y,m,d)							// if user didn't cancel
         this.dateValue = new date(y,m-1,d)		// store return value
      endif
      return not empty( this.dateValue )

   function release
   	release object this
      return

endclass

class GetDirectoryDialog
/*
	wrapper for a custom GetDirectory dialog from CmnDlgs.dll

   Implementation:
   	set procedure to CmnDlgs.cc additive
      d = new GetDirectoryDialog()
      d.titleText := 'Where to you want to put it?'
      d.startDir := 'C:\Windows'
      if d.execute()
      	cMyDirectory = d.directory
      else
      	// user canceled
      endif
      d.release()

*/
	protect Init

	this.startDir = set('directory')		// default starting point
	this.titleText = 'Choose a folder'	// default title bar text
   this.allowCreate = true					// can user create a new folder
   this.directory = ''						// result stored here
	this.Init()									// prototype the extern

   function execute
   	// returns true if the user chose a directory
      // or false if user canceled.  Directory chosen
      // is stored in the Directory property.
   	with ( this )
      	directory := GetDirName( startDir, titleText, allowCreate)
      endwith
      return not empty( this.directory )

   function Init
      if type('GetDirName') # 'FP'
         extern CSTRING GetDirName( CSTRING, CSTRING, CLOGICAL ) CmnDlgs
      endif

   function release
   	release object this
      return

endclass

